#!/bin/env python
# -*- coding: utf-8 -*-

# ------------------------------------------------------------
# configure_dialog.pyw
# 
# let the user configure the opensecurity system
#
# Autor: Oliver Maurhart, <oliver.maurhart@ait.ac.at>
#
# Copyright (C) 2014 AIT Austrian Institute of Technology
# AIT Austrian Institute of Technology GmbH
# Donau-City-Strasse 1 | 1220 Vienna | Austria
# http://www.ait.ac.at
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation version 2.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, 
# Boston, MA  02110-1301, USA.
# ------------------------------------------------------------


# ------------------------------------------------------------
# for the record:
#
#   This script uses a QTimer (in a busy loop) for checking
#   changes to the log file watched.
#
#   Yes, this is sure f--g ugly.
#
#   Thing is: there is no real good cross-platform in
#   doing this nicely. And here we solve this problem
#   neither. See doc in update_progress method.
#
#   There is no python native but external packages
#   using pip with different qualities (and struggling).
#
#   Qt and as such PyQt offer the QFileSystemWatcher class
#   which *could* solve this isse but
#
#       a) this class is somehow buggy and unstable
#       b) deprecated as of Qt 5.1
#
#   ... and this ain't a beauty contest either, is it?
# ------------------------------------------------------------


# ------------------------------------------------------------
# imports

import json
import os
import subprocess
import sys
import urllib2

from PyQt4 import QtCore
from PyQt4 import QtGui

from ui_ConfigureDialog import Ui_ConfigureDialog 
from about_dialog import AboutDialog


# ------------------------------------------------------------
# code


class ConfigureDialog(QtGui.QDialog):

    """A dialog which lets the user configure the OpenSecurity Subsystem"""

    def __init__(self):

        QtGui.QDialog.__init__(self)

        # setup the user interface
        self.ui = Ui_ConfigureDialog()
        self.ui.setupUi(self)

        # monospace font fix for windows systems
        font = QtGui.QFont("Monospace [Courier]")
        font.setPointSize(8);
        font.setStyleHint(QtGui.QFont.TypeWriter)
        font.setStyleStrategy(QtGui.QFont.PreferAntialias)
        self.ui.edtProgress.setFont(font)
        self.ui.edtLog.setFont(font)
    
        self.ui.edtStatus.setText('<not evaluated yet>')
        self.ui.edtVersion.setText('<not evaluated yet>')
        self.ui.edtStatus.setEnabled(False);
        self.ui.edtVersion.setEnabled(False);

        # local members
        self._about_dialog = AboutDialog()
        self._file_watched_name = ''
        self._file_watched_size = 0
        self._file_watcher = QtCore.QTimer(self)
        self._service_log_file = ''
        self._service_log_size = 0

        # connectors
        self.ui.btnAbout.clicked.connect(self.clicked_about)
        self.ui.btnClose.clicked.connect(self.accept)
        self.ui.btnDownload.clicked.connect(self.clicked_download)
        self.ui.btnExplorer.clicked.connect(self.clicked_explorer)
        self.ui.btnImport.clicked.connect(self.clicked_import)
        self.ui.btnRefresh.clicked.connect(self.clicked_refresh)
        self._file_watcher.timeout.connect(self.update_progress)

        # call first refresh immediately
        QtCore.QTimer.singleShot(0, self.clicked_refresh)
        self._file_watcher.start(100)


    def clicked_about(self):

        """About button has been clicked."""
        self._about_dialog.exec_()


    def clicked_download(self):

        """Download button has been clicked."""

        # remove old stuff
        self._file_watched_name = ''
        self._file_watched_size = 0
        self.ui.edtProgress.clear()

        try:

            # get general server info
            j = json.load(urllib2.urlopen('http://127.0.0.1:8080/fetch_initial_image'))
            self._file_watched_name = str(j['fetch_log'])

        except:
            pass


    def clicked_explorer(self):

        """Explorer button has been clicked."""
        if sys.platform == 'win32' or sys.platform == 'cygwin':
            subprocess.Popen([os.path.join(os.environ['WINDIR'], 'explorer.exe'), self._service_log_path])


    def clicked_import(self):

        """Import button has been clicked."""

        # remove old stuff
        self._file_watched_name = ''
        self._file_watched_size = 0
        self.ui.edtProgress.clear()

        try:

            # get general server info
            j = json.load(urllib2.urlopen('http://127.0.0.1:8080/init'))
            self._file_watched_name = str(j['init_log'])

        except:
            pass


    def clicked_refresh(self):

        """Refresh button has been clicked."""
        self.ui.edtStatus.setText('<not evaluated yet>')
        self.ui.edtVersion.setText('<not evaluated yet>')
        self.ui.edtStatus.setEnabled(False);
        self.ui.edtVersion.setEnabled(False);
        self.ui.tvTemplate.clear()

        try:

            # get general server info
            j = json.load(urllib2.urlopen('http://127.0.0.1:8080'))
            self.ui.edtStatus.setText(j['os_server']['status message'])
            self.ui.edtStatus.setEnabled(True)
            self.ui.edtVersion.setText(j['os_server']['version'])
            self.ui.edtVersion.setEnabled(True)
            self._service_log_path = j['os_server']['current log folder']
            self._service_log_file = os.path.join(self._service_log_path, 'OpenSecurity.log')
            self._service_log_size = 0

            # get initial template info
            j = json.load(urllib2.urlopen('http://127.0.0.1:8080/initial_image'))
            i = QtGui.QTreeWidgetItem(self.ui.tvTemplate)
            i.setText(0, j['initial_template']['name'])
            i.setText(1, QtCore.QDateTime.fromTime_t(int(j['initial_template']['date'])).toString(QtCore.Qt.SystemLocaleShortDate))
            i.setText(2, str(j['initial_template']['size']))
            i.setText(3, j['initial_template']['path'])
            
        except:
            pass

        # trigger machine update
        self.update_machines()


    def update_machines(self):

        """Update the machines view"""
        self.ui.tvMachines.clear()
        self._icon_machine = QtGui.QIcon()
        self._icon_machine.addPixmap(QtGui.QPixmap(":/opensecurity/gfx/cpu.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self._icon_property = QtGui.QIcon()
        self._icon_property.addPixmap(QtGui.QPixmap(":/opensecurity/gfx/configure.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self._icon_network = QtGui.QIcon()
        self._icon_network.addPixmap(QtGui.QPixmap(":/opensecurity/gfx/network-workgroup.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)

        try:

            # get machines
            machines = json.load(urllib2.urlopen('http://127.0.0.1:8080/sdvms'))
            for m in machines:
                i = QtGui.QTreeWidgetItem(self.ui.tvMachines)
                i.setIcon(0, self._icon_machine)
                i.setText(0, m)
                i.setIcon(1, self._icon_network)
                i.setText(1, machines[m])


                properties = json.load(urllib2.urlopen('http://127.0.0.1:8080/sdvms/' + m))
                for p in properties:
                    j = QtGui.QTreeWidgetItem(i)
                    j.setIcon(0, self._icon_property)
                    j.setText(0, p)
                    j.setText(1, properties[p])

                j.setIcon(0, self._icon_network)
                j.setText(0, 'ip')
                j.setText(1, machines[m])


        except:
            pass



    def update_progress(self):
        
        """Our log file has changed, update ui"""

        # we could use os.stat(PATH).st_mtime ...
        # 
        # ... but Windows does not update it frequently. -.- 
        #
        # Running a Cygwin Bash Shell script in the background
        # does not update mtime of the redirected stderr file.
        #
        # This is terrible. 

        # first the service log
        if self._service_log_file != '':
            file_size = os.stat(self._service_log_file).st_size
            if file_size > self._service_log_size:
                f = open(self._service_log_file, 'r')
                f.seek(self._service_log_size)
                self.ui.edtLog.appendPlainText(f.read())
                self._service_log_size = file_size
        
        # now any processing log (download or import)
        if self._file_watched_name == '':
            return

        file_size = os.stat(self._file_watched_name).st_size
        if self._file_watched_size >= file_size:
            return

        f = open(self._file_watched_name, 'r')
        self.ui.edtProgress.setPlainText(f.read())
        self.ui.edtProgress.verticalScrollBar().setValue(self.ui.edtProgress.verticalScrollBar().maximum())
        self._file_watched_size = file_size


if __name__ == "__main__":
    a = QtGui.QApplication(sys.argv)
    d = ConfigureDialog()
    d.show()
    sys.exit(a.exec_())     

