#!/bin/env python
# -*- coding: utf-8 -*-

# ------------------------------------------------------------
# launch_dialog.pyw
# 
# let the user launch an app in a vm
#
# Autor: Oliver Maurhart, <oliver.maurhart@ait.ac.at>
#
# Copyright (C) 2014 AIT Austrian Institute of Technology
# AIT Austrian Institute of Technology GmbH
# Donau-City-Strasse 1 | 1220 Vienna | Austria
# http://www.ait.ac.at
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation version 2.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, 
# Boston, MA  02110-1301, USA.
# ------------------------------------------------------------


# ------------------------------------------------------------
# imports

import sys

from PyQt4 import QtCore
from PyQt4 import QtGui

from ui_LaunchDialog import Ui_LaunchDialog 
from about_dialog import AboutDialog


# ------------------------------------------------------------
# code


class LaunchDialog(QtGui.QDialog):

    """A dialog for letting the user launch an arbitrary app in a vm"""

    def __init__(self, vms, apps):

        QtGui.QDialog.__init__(self)

        # setup the user interface
        self.ui = Ui_LaunchDialog()
        self.ui.setupUi(self)
        for v in vms: 
            self.ui.cmbVM.addItem(v)
        for a in apps: 
            self.ui.cmbApplication.addItem(a)
    
        # local members
        self._about_dialog = AboutDialog()

        # connectors
        self.ui.btnAbout.clicked.connect(self.clicked_about)
        self.ui.btnCancel.clicked.connect(self.reject)
        self.ui.btnOk.clicked.connect(self.clicked_ok)

        # positionate ourself central
        screen = QtGui.QDesktopWidget().screenGeometry()
        size = self.geometry()
        self.move((screen.width() - size.width()) / 2, (screen.height() - size.height()) / 2)
 

    def clicked_about(self):

        """About button has been clicked."""
        self._about_dialog.show()


    def clicked_ok(self):
        
        """Ok button has been clicked."""
        sys.stdout.write('{ ')
        sys.stdout.write('"application": "')
        sys.stdout.write(self.app)
        sys.stdout.write('", ')
        sys.stdout.write('"vm": "')
        sys.stdout.write(self.vm)
        sys.stdout.write('" ')
        sys.stdout.write('}\n')
        self.accept()


    @property
    def app(self):

        """The application of the user."""
        return str(self.ui.cmbApplication.currentText())
 

    @property
    def vm(self):

        """The application of the user."""
        return str(self.ui.cmbVM.currentText())
 

if __name__ == "__main__":
    a = QtGui.QApplication(sys.argv)
    d = LaunchDialog(['SecurityDVM0', 'SecurityDVM1'], ['/usr/bin/xterm', '/usr/bin/xclock', '/usr/bin/xeyes'])
    d.show()
    sys.exit(a.exec_())     

